/*
 *  Copyright (c) 2017, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied. See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 *  WSO2 API Manager - Publisher API
 *  This specifies a **RESTful API** for WSO2 **API Manager** - Publisher.  Please see [full swagger definition](https://raw.githubusercontent.com/wso2/carbon-apimgt/v6.0.4/components/apimgt/org.wso2.carbon.apimgt.rest.api.publisher/src/main/resources/publisher-api.yaml) of the API which is written using [swagger 2.0](http://swagger.io/) specification.
 *
 *  OpenAPI spec version: 0.10.0
 *  Contact: architecture@wso2.com
 *
 *  NOTE: This class is auto generated by the swagger code generator program.
 *  https://github.com/swagger-api/swagger-codegen.git
 *  Do not edit the class manually.
 *
 */

package org.wso2.carbon.apimgt.rest.integration.tests.api.publisher;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.testng.Assert;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.wso2.carbon.apimgt.rest.integration.tests.publisher.api.APIIndividualApi;
import org.wso2.carbon.apimgt.rest.integration.tests.publisher.api.APICollectionApi;
import org.wso2.carbon.apimgt.rest.integration.tests.publisher.ApiException;
import org.wso2.carbon.apimgt.rest.integration.tests.publisher.model.API;
import org.wso2.carbon.apimgt.rest.integration.tests.publisher.model.Sequence;
import org.wso2.carbon.apimgt.rest.integration.tests.publisher.model.APIBusinessInformation;
import org.wso2.carbon.apimgt.rest.integration.tests.publisher.model.APICorsConfiguration;
import org.wso2.carbon.apimgt.rest.integration.tests.publisher.model.LifecycleState;
import org.wso2.carbon.apimgt.rest.integration.tests.publisher.model.APIList;

import java.io.File;

import java.util.ArrayList;

import org.testng.annotations.Test;

/**
 * API tests for APIIndividualApi
 */
public class APIIndividualApiIT {

    private final APIIndividualApi api = new APIIndividualApi();
    private final APICollectionApi apiSetup = new APICollectionApi();
    private final TestUtils testUtils = new TestUtils();
    private String APIID = null;

    @BeforeClass
    public void beforeClass() throws ApiException {

        API body = new API();
        body.setName("Api101");
        body.setContext("api101");
        body.setVersion("1.0.0");
        body.setDescription("This is the api description");
        body.setProvider("admin");
        body.setWsdlUri("https://graphical.weather.gov/xml/SOAP_server/ndfdXMLserver.php?wsdl");
        body.setLifeCycleStatus("CREATED");
        body.setTransport(new ArrayList<String>() {{
            add("http");
        }});
        body.setCacheTimeout(100);
        body.setPolicies(new ArrayList<String>() {{
            add("Unlimited");
        }});
        body.setVisibility(API.VisibilityEnum.PUBLIC);
        body.setTags(new ArrayList<String>());
        body.setVisibleRoles(new ArrayList<String>());
        body.setVisibleTenants(new ArrayList<String>());
        body.setSequences(new ArrayList<Sequence>());
        body.setBusinessInformation(new APIBusinessInformation());
        body.setCorsConfiguration(new APICorsConfiguration());
        API response = apiSetup.apisPost(body);
        APIID = response.getId();
    }

    @Test(description = "This testcase verifies if created APIS can be viewed through Publisher REST API.")
    public void apisApiIdGetTest() throws ApiException {
        String apiId = APIID;
        String ifNoneMatch = null;
        String ifModifiedSince = null;
        try {
            API response = api.apisApiIdGet(apiId, ifNoneMatch, ifModifiedSince);
            Assert.assertEquals(response.getName(), "Api101", "API name mismatch");
            Assert.assertEquals(response.getContext(), "api101", "API context mismatch");
            Assert.assertEquals(response.getVersion(), "1.0.0", "API version mismatch");
            Assert.assertEquals(response.getId(), apiId, "API id mismatch");
            Assert.assertEquals(response.getDescription(), "This is the api description", "API discription mismatch");
            Assert.assertEquals(response.getLifeCycleStatus(), "Created", "API lifecycle status mismatch");
        } catch (ApiException apiException) {
            System.out.println(apiException.getCode());
            System.out.println(apiException.getResponseBody());
        }
    }

    @Test(description = "This testcase verifies if an api can be retrieved when given an invalid api Id - Negative")
    public void apisApiIdGetTest_FailureTest() throws ApiException {
        try {
            String apiId = "InvalidApiId";
            String ifNoneMatch = null;
            String ifModifiedSince = null;
            api.apisApiIdGet(apiId, ifNoneMatch, ifModifiedSince);
        } catch (ApiException ae) {
            JsonParser parser = new JsonParser();
            JsonObject responseBody = (JsonObject) parser.parse(ae.getResponseBody());
            String errorMsg = responseBody.get("message").getAsString();
            Assert.assertEquals(errorMsg, "API not found", "Response message mismatch");
            Assert.assertEquals(ae.getCode(), 404, "Response code mismatch");
        }
    }

    @Test(description = "This testcase verifies if an existing API can be updated successfully. ")
    public void apisApiIdPutTest() throws ApiException {

        String newApiDescription = "New API Description";
        String apiId = APIID;
        try {
            API body = api.apisApiIdGet(apiId, null, null);
            body.setDescription(newApiDescription);
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            API response = api.apisApiIdPut(apiId, body, ifMatch, ifUnmodifiedSince);
            Assert.assertEquals(response.getDescription(), newApiDescription, "description mismatch");
        } catch (ApiException apiException) {
            System.out.println(apiException.getCode());
            System.out.println(apiException.getResponseBody());
            assert false;
        }
    }

    /**
     * FAILS
     * PLease refer https://github.com/wso2/product-apim/issues/1626
     */
    @Test(description = "This testcase verifies if an existing API can be updated successfully.",
          enabled = false)
    public void apisApiIdPutTest_FailureTest_400() throws ApiException {

        String apiId = APIID;
        try {
            API body = api.apisApiIdGet(apiId, null, null);
            body.setName("Name changed");
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            api.apisApiIdPut(apiId, body, ifMatch, ifUnmodifiedSince);

        } catch (ApiException apiException) {
            int responseCode = apiException.getCode();
            Assert.assertEquals(responseCode, 400, "response code mismatch");
        }
    }

    @Test(description = "This testcase verifies if an existing API can be updated when an invalid APIID is provided. ")
    public void apisApiIdPutTest_NF() throws ApiException {
        try {
            String newApiDescription = "New API Description";
            String apiId = "invalidID";
            API body = api.apisApiIdGet(APIID, null, null);
            body.setDescription(newApiDescription);
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            api.apisApiIdPut(apiId, body, ifMatch, ifUnmodifiedSince);
        } catch (ApiException ae) {
            JsonParser parser = new JsonParser();
            JsonObject responseBody = (JsonObject) parser.parse(ae.getResponseBody());
            String errorMsg = responseBody.get("message").getAsString();
            Assert.assertEquals(errorMsg, "API not found", "Response message mismatch");
            Assert.assertEquals(ae.getCode(), 404, "status code mismatch");
        }
    }

    /**
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false,
          description = "This testcase verifies if API's swagger definition can be retrieved. ")
    public void apisApiIdSwaggerGetTest() throws ApiException {
        String apiId = APIID;
        String ifModifiedSince = null;
        String ifNoneMatch = null;
        api.apisApiIdSwaggerGet(apiId, ifNoneMatch, ifModifiedSince);
    }

    /* FAILS
     * please refer https://github.com/wso2/product-apim/issues/1611
    */
    @Test(enabled = false,
          description = "This testcase verifies if API's swagger definition can be retrieved when an invalid APIID is provided.")
    public void apisApiIdSwaggerGetTest_NF() throws ApiException {
        try {
            String apiId = "invalidId";
            String ifModifiedSince = null;
            String ifNoneMatch = null;
            api.apisApiIdSwaggerGet(apiId, ifNoneMatch, ifModifiedSince);
        } catch (ApiException ae) {
            Assert.assertEquals(ae.getCode(), 404, "status code mismatch");
            JsonParser parser = new JsonParser();
            JsonObject responseBody = (JsonObject) parser.parse(ae.getResponseBody());
            String errorMsg = responseBody.get("message").getAsString();
            Assert.assertEquals(errorMsg, "API not found", "Response message mismatch");
        }
    }

    /**
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false,
          description = "This testcase verifies if swagger definition can be updated for an existing api. ")
    public void apisApiIdSwaggerPutTest() throws ApiException {
        String apiId = APIID;
        String ifMatch = null;
        String endPointId = "{\\r\\n\\t\\\"swagger\\\": \\\"2.0\\\",\\r\\n\\t\\\"info\\\": {\\r\\n\\t\\t\\\"version\\\": \\\"1.0.0\\\",\\r\\n\\t\\t\\\"title\\\": \\\"TestApi\\\",\\r\\n\\t\\t\\\"contact\\\": {}\\r\\n\\t},\\r\\n\\t\\\"paths\\\": {\\r\\n\\t\\t\\\"\\/\\\": {\\r\\n\\t\\t\\t\\\"get\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"get\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"head\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"head\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"post\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"post\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [{\\r\\n\\t\\t\\t\\t\\t\\\"in\\\": \\\"body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"name\\\": \\\"Payload\\\",\\r\\n\\t\\t\\t\\t\\t\\\"description\\\": \\\"Request Body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"required\\\": false,\\r\\n\\t\\t\\t\\t\\t\\\"schema\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"properties\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\\"payload\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\t\\\"type\\\": \\\"string\\\"\\r\\n\\t\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"put\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"put\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [{\\r\\n\\t\\t\\t\\t\\t\\\"in\\\": \\\"body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"name\\\": \\\"Payload\\\",\\r\\n\\t\\t\\t\\t\\t\\\"description\\\": \\\"Request Body1\\\",\\r\\n\\t\\t\\t\\t\\t\\\"required\\\": false,\\r\\n\\t\\t\\t\\t\\t\\\"schema\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"properties\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\\"payload\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\t\\\"type\\\": \\\"string\\\"\\r\\n\\t\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"delete\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"delete\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"patch\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"patch\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [{\\r\\n\\t\\t\\t\\t\\t\\\"in\\\": \\\"body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"name\\\": \\\"Payload\\\",\\r\\n\\t\\t\\t\\t\\t\\\"description\\\": \\\"Request Body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"required\\\": false,\\r\\n\\t\\t\\t\\t\\t\\\"schema\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"properties\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\\"payload\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\t\\\"type\\\": \\\"string\\\"\\r\\n\\t\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t}\\r\\n\\t\\t}\\r\\n\\t}\\r\\n}";
        String ifUnmodifiedSince = null;
        api.apisApiIdSwaggerPut(apiId, endPointId, ifMatch, ifUnmodifiedSince);
    }

    /**
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false,
          description = "This testcase verifies if swagger definition can be updated for an existing api. ")
    public void apisApiIdSwaggerPutTest_FailureTest_404() throws ApiException {
        String apiId = APIID;
        String ifMatch = null;
        String endPointId = "{\\r\\n\\t\\\"swagger\\\": \\\"2.0\\\",\\r\\n\\t\\\"info\\\": {\\r\\n\\t\\t\\\"version\\\": \\\"1.0.0\\\",\\r\\n\\t\\t\\\"title\\\": \\\"TestApi\\\",\\r\\n\\t\\t\\\"contact\\\": {}\\r\\n\\t},\\r\\n\\t\\\"paths\\\": {\\r\\n\\t\\t\\\"\\/\\\": {\\r\\n\\t\\t\\t\\\"get\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"get\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"head\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"head\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"post\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"post\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [{\\r\\n\\t\\t\\t\\t\\t\\\"in\\\": \\\"body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"name\\\": \\\"Payload\\\",\\r\\n\\t\\t\\t\\t\\t\\\"description\\\": \\\"Request Body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"required\\\": false,\\r\\n\\t\\t\\t\\t\\t\\\"schema\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"properties\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\\"payload\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\t\\\"type\\\": \\\"string\\\"\\r\\n\\t\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"put\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"put\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [{\\r\\n\\t\\t\\t\\t\\t\\\"in\\\": \\\"body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"name\\\": \\\"Payload\\\",\\r\\n\\t\\t\\t\\t\\t\\\"description\\\": \\\"Request Body1\\\",\\r\\n\\t\\t\\t\\t\\t\\\"required\\\": false,\\r\\n\\t\\t\\t\\t\\t\\\"schema\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"properties\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\\"payload\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\t\\\"type\\\": \\\"string\\\"\\r\\n\\t\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"delete\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"delete\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t},\\r\\n\\t\\t\\t\\\"patch\\\": {\\r\\n\\t\\t\\t\\t\\\"operationId\\\": \\\"patch\\\",\\r\\n\\t\\t\\t\\t\\\"parameters\\\": [{\\r\\n\\t\\t\\t\\t\\t\\\"in\\\": \\\"body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"name\\\": \\\"Payload\\\",\\r\\n\\t\\t\\t\\t\\t\\\"description\\\": \\\"Request Body\\\",\\r\\n\\t\\t\\t\\t\\t\\\"required\\\": false,\\r\\n\\t\\t\\t\\t\\t\\\"schema\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"properties\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\\"payload\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\t\\t\\\"type\\\": \\\"string\\\"\\r\\n\\t\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}],\\r\\n\\t\\t\\t\\t\\\"responses\\\": {\\r\\n\\t\\t\\t\\t\\t\\\"200\\\": {\\r\\n\\t\\t\\t\\t\\t\\t\\\"description\\\": \\\"OK\\\"\\r\\n\\t\\t\\t\\t\\t}\\r\\n\\t\\t\\t\\t}\\r\\n\\t\\t\\t}\\r\\n\\t\\t}\\r\\n\\t}\\r\\n}";
        String ifUnmodifiedSince = null;
        api.apisApiIdSwaggerPut(apiId, endPointId, ifMatch, ifUnmodifiedSince);
    }

    /**
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false,
          description = "This testcase verifies if thumbnail image can be uploaded")
    public void apisApiIdThumbnailPostTest() throws ApiException {
        String apiId = APIID;
        File file = new File(getClass().getResource("/img1.jpg").getFile());
        String ifMatch = null;
        String ifUnmodifiedSince = null;
        api.apisApiIdThumbnailPost(apiId, file, ifMatch, ifUnmodifiedSince);
    }

    /**
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     ****/
    @Test(enabled = false,
          description = "This testcase verifies if thumbnail image can be uploaded for a given API")
    public void apisApiIdThumbnailGetTest() throws ApiException {
        String apiId = APIID;
        String accept = null;
        String ifNoneMatch = null;
        String ifModifiedSince = null;
        api.apisApiIdThumbnailGet(apiId, ifNoneMatch, ifModifiedSince);
    }

    /**
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false,
          description = "This testcase verifies if API ID's wsdl document can be viewed. ")
    public void apisApiIdWsdlGetTest() throws ApiException {
        String apiId = APIID;
        String ifNoneMatch = null;
        String ifModifiedSince = null;
        api.apisApiIdWsdlGet(apiId, ifNoneMatch, ifModifiedSince);
    }

    @Test(enabled = true,
          description = "This testcase verifies if API ID's wsdl document can be viewed. ")
    public void apisApiIdWsdlGetTest_failureTest_404() throws ApiException {
        String apiId = "invalidId";
        String ifNoneMatch = null;
        String ifModifiedSince = null;
        try {
            api.apisApiIdWsdlGet(apiId, ifNoneMatch, ifModifiedSince);
        } catch (ApiException apiException) {
            int statusCode = apiException.getCode();
            Assert.assertEquals(statusCode, 404, "Status code mismatch");
        }

    }

    /**
     * FAILS
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false,
          description = "This testcase verifies if API ID's wsdl document can be updated. ")
    public void apisApiIdWsdlPutTest() throws ApiException {
        String apiId = APIID;
        File file = new File(getClass().getResource("/Phone.wsdl").getFile());
        String ifMatch = null;
        String ifUnmodifiedSince = null;
        api.apisApiIdWsdlPut(apiId, file, ifMatch, ifUnmodifiedSince);
        api.apisApiIdWsdlGet(apiId, null, null);
    }

    /**
     * FAILS
     * please refer https://github.com/wso2/product-apim/issues/1627
     **/
    @Test(enabled = false,
          description = "This testcase verifies if API ID's wsdl document can be updated. ")
    public void apisApiIdWsdlPutTest_failureTest_404() throws ApiException {
        String apiId = "invalidApi";
        File file = new File(getClass().getResource("/Phone.wsdl").getFile());
        String ifMatch = null;
        String ifUnmodifiedSince = null;
        try {
            api.apisApiIdWsdlPut(apiId, file, ifMatch, ifUnmodifiedSince);
        } catch (ApiException apiException) {
            int statusCode = apiException.getCode();
            Assert.assertEquals(statusCode, 404, "Status code mismatch");
        }

    }

    @Test(enabled = true,
          description = "This testcase verifies if API ID's wsdl document can be updated. ")
    public void apisApiIdWsdlPutTest_failureTest_400() throws ApiException {
        String apiId = APIID;
        File file = new File(getClass().getResource("/img1.jpg").getFile());
        String ifMatch = null;
        String ifUnmodifiedSince = null;
        try {
            api.apisApiIdWsdlPut(apiId, file, ifMatch, ifUnmodifiedSince);
        } catch (ApiException apiException) {
            int statusCode = apiException.getCode();
            Assert.assertEquals(statusCode, 400, "Status code mismatch");
        }

    }

    @Test(enabled = true,
          description = "This testcase verifies if a new API version can be created for a given existing API.")
    public void apisCopyApiPostTest_FailureTest_404() throws ApiException {
        String newVersion = "2.0.1";
        String apiId = "invalidId";
        try {
            api.apisCopyApiPost(newVersion, apiId);
            assert false;
        } catch (ApiException ae) {
            int statusCode = ae.getCode();
            Assert.assertEquals(statusCode, 404, "status code mismatch");
        }
    }

    /**
     * FAILS
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false,
          description = "This testcase verifies if a new API version can be created for a given existing API.")
    public void apisCopyApiPostTest_() throws ApiException {
        String newVersion = "2.0.1";
        String apiId = APIID;
        api.apisCopyApiPost(newVersion, apiId);

    }

    @Test(description = "Verify if specific api's lifecycle state can be retrieved.")
    public void apisApiIdLifecycleGetTest() throws ApiException {
        String apiId = APIID;
        String ifNoneMatch = null;
        String ifModifiedSince = null;

        LifecycleState response = api.apisApiIdLifecycleGet(apiId, ifNoneMatch, ifModifiedSince);
        String lifeCycleState = response.getState();
        Assert.assertEquals(lifeCycleState, "Created", "Api lifecycle state mismatch");

    }

    @Test(description = "Verify if specific api's lifecycle state can be retrieved when an invalid Api Id is provided.")
    public void apisApiIdLifecycleGetTest_NF() throws ApiException {
        try {
            String apiId = "invalidId";
            String ifNoneMatch = null;
            String ifModifiedSince = null;
            api.apisApiIdLifecycleGet(apiId, ifNoneMatch, ifModifiedSince);
        } catch (ApiException ae) {
            JsonParser parser = new JsonParser();
            JsonObject responseBody = (JsonObject) parser.parse(ae.getResponseBody());
            String errorMsg = responseBody.get("message").getAsString();
            Assert.assertEquals(errorMsg, "API not found", "Response message mismatch");
            Assert.assertEquals(ae.getCode(), 404, "status code mismatch");
        }
    }

    /**
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false,
          description = "This testcase verifies if state change history of an api life cycle can be retrieved.")
    public void apisApiIdLifecycleHistoryGetTest() throws ApiException {
        String apiId = APIID;
        String ifNoneMatch = null;
        String ifModifiedSince = null;
        api.apisApiIdLifecycleHistoryGet(apiId, ifNoneMatch, ifModifiedSince);

    }

    @Test(enabled = true,
          description = "This testcase verifies if state change history of an api life cycle can be retrieved.")
    public void apisApiIdLifecycleHistoryGetTest_failureTest() throws ApiException {
        String apiId = "invalidApiId";
        String ifNoneMatch = null;
        String ifModifiedSince = null;
        try {
            api.apisApiIdLifecycleHistoryGet(apiId, ifNoneMatch, ifModifiedSince);
        } catch (ApiException apiException) {
            int statusCode = apiException.getCode();
            Assert.assertEquals(statusCode, 404, "Status code mismatch");
        }
    }

    /**
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false)
    public void apisApiIdGatewayConfigGetTest() throws ApiException {
        String apiId = APIID;
        String ifNoneMatch = null;
        String ifModifiedSince = null;
        api.apisApiIdGatewayConfigGet(apiId, ifNoneMatch, ifModifiedSince);
    }

    @Test
    public void apisApiIdGatewayConfigGetTest_failureTest() throws ApiException {
        String apiId = "invalidId";
        String ifNoneMatch = null;
        String ifModifiedSince = null;
        try {
            api.apisApiIdGatewayConfigGet(apiId, ifNoneMatch, ifModifiedSince);
            assert false;
        } catch (ApiException apiException) {
            int statusCode = apiException.getCode();
            Assert.assertEquals(statusCode, 404, "Status code mismatch");
        }
    }

    /**
     * please refer https://github.com/wso2/product-apim/issues/1575
     * due to this issue response cannot be taken to be compared. Therefore disabling this test until the issue is fixed.
     **/
    @Test(enabled = false)
    public void apisApiIdGatewayConfigPutTest() throws ApiException {
        String apiId = APIID;
        String gatewayConfig = null;
        String ifMatch = null;
        String ifUnmodifiedSince = null;
        api.apisApiIdGatewayConfigPut(apiId, gatewayConfig, ifMatch, ifUnmodifiedSince);
    }

    @Test(description = "This testcase verifies if change of an api life cycle can be done.")
    public void apisChangeLifecyclePostTest() throws ApiException {
        String lifecycleChecklist = null;
        String ifMatch = null;
        String ifUnmodifiedSince = null;
        API response;
        String apiId = testUtils.createApi("API-120", "1.0.0", "API-120");

        // Created --> Prototyped
        api.apisChangeLifecyclePost("Prototyped", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        response = api.apisApiIdGet(apiId, null, null);
        Assert.assertEquals("Prototyped", response.getLifeCycleStatus(), "API status mismatch Created --> Prototyped");

        // Prototyped --> Created
        api.apisChangeLifecyclePost("Created", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        response = api.apisApiIdGet(apiId, null, null);
        Assert.assertEquals("Created", response.getLifeCycleStatus(), "API status mismatch Prototyped --> Created");

        //Created --> Published
        api.apisChangeLifecyclePost("Published", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        response = api.apisApiIdGet(apiId, null, null);
        Assert.assertEquals("Published", response.getLifeCycleStatus(), "API status mismatch Created --> Published");

        //Published --> Created
        api.apisChangeLifecyclePost("Created", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        response = api.apisApiIdGet(apiId, null, null);
        Assert.assertEquals("Created", response.getLifeCycleStatus(), "API status mismatch Published --> Created");

        api.apisChangeLifecyclePost("Published", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);

        // Published --> Maintenance
        api.apisChangeLifecyclePost("Maintenance", apiId, lifecycleChecklist, null, null);
        response = api.apisApiIdGet(apiId, null, null);
        Assert.assertEquals("Maintenance", response.getLifeCycleStatus(),
                "API status mismatch Published --> Maintenance");

        // Maintenance --> Published
        api.apisChangeLifecyclePost("Published", apiId, lifecycleChecklist, null, null);
        response = api.apisApiIdGet(apiId, null, null);
        Assert.assertEquals("Published", response.getLifeCycleStatus(),
                "API status mismatch Maintenance --> Published");

        // Published --> Deprecated
        api.apisChangeLifecyclePost("Deprecated", apiId, lifecycleChecklist, null, null);
        response = api.apisApiIdGet(apiId, null, null);
        Assert.assertEquals("Deprecated", response.getLifeCycleStatus(),
                "API status mismatch Published --> Deprecated");

        //Deprecated --> Retired
        api.apisChangeLifecyclePost("Retired", apiId, null, null, null);
        response = api.apisApiIdGet(apiId, null, null);
        Assert.assertEquals("Retired", response.getLifeCycleStatus(), "API status mismatch Deprecated --> Retired");

        String apiId2 = testUtils.createApi("API-121", "1.0.0", "API-121");
        api.apisChangeLifecyclePost("Published", apiId2, lifecycleChecklist, null, null);
        api.apisChangeLifecyclePost("Maintenance", apiId2, lifecycleChecklist, null, null);

        // Maintenance --> Deprecated
        api.apisChangeLifecyclePost("Deprecated", apiId2, lifecycleChecklist, null, null);
        response = api.apisApiIdGet(apiId2, null, null);
        Assert.assertEquals("Deprecated", response.getLifeCycleStatus(),
                "API status mismatch Maintenance --> Deprecated");

        testUtils.deleteApi();
    }

    @Test
    public void apisChangeLifecyclePostTest_FailureTest_404() throws ApiException {
        String lifecycleChecklist = null;
        String ifMatch = null;
        String ifUnmodifiedSince = null;
        String apiId = "invaliApiId";
        try {
            api.apisChangeLifecyclePost("Prototyped", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        } catch (ApiException apiException) {
            int statusCode = apiException.getCode();
            Assert.assertEquals(statusCode, 404, "Status code mismatch");
        }
    }

    /* FAILS
     * please refer
     * https://github.com/wso2/product-apim/issues/1610
     */
    @Test(description = "This testcase verifies if change of an api life cycle can be done - NF",
          enabled = false)
    public void apisChangeLifecyclePostTest_FailureTest_400() throws ApiException {
        try {
            String lifecycleChecklist = null;
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            String apiId = testUtils.createApi("API-120", "1.0.0", "API-120");

            // Created --> Maintenance
            api.apisChangeLifecyclePost("Maintenance", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        } catch (ApiException ae) {
            Assert.assertEquals(ae.getCode(), 400, "Response code mismatch. Created --> Maintenance");
            testUtils.deleteApi();
        }

        try {
            String lifecycleChecklist = null;
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            String apiId = testUtils.createApi("API-120", "1.0.0", "API-120");

            // Created --> Deprecated
            api.apisChangeLifecyclePost("Deprecated", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        } catch (ApiException ae) {
            Assert.assertEquals(ae.getCode(), 400, "Response code mismatch. Created --> Deprecated");
            testUtils.deleteApi();
        }

        try {
            String lifecycleChecklist = null;
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            String apiId = testUtils.createApi("API-120", "1.0.0", "API-120");

            // Prototyped --> Maintenance
            api.apisChangeLifecyclePost("Prototyped", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
            api.apisChangeLifecyclePost("Maintenance", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        } catch (ApiException ae) {
            Assert.assertEquals(ae.getCode(), 400, "Response code mismatch. Prototyped --> Maintenance");
            testUtils.deleteApi();
        }

        try {
            String lifecycleChecklist = null;
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            String apiId = testUtils.createApi("API-120", "1.0.0", "API-120");

            // Created --> Retired
            api.apisChangeLifecyclePost("Retired", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        } catch (ApiException ae) {
            Assert.assertEquals(ae.getCode(), 400, "Response code mismatch. Created --> Retired");
            testUtils.deleteApi();
        }

        try {
            String lifecycleChecklist = null;
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            String apiId = testUtils.createApi("API-120", "1.0.0", "API-120");

            // Deprecated --> Maintenance
            api.apisChangeLifecyclePost("Published", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
            api.apisChangeLifecyclePost("Deprecated", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
            api.apisChangeLifecyclePost("Maintenance", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        } catch (ApiException ae) {
            Assert.assertEquals(ae.getCode(), 400, "Response code mismatch. Deprecated --> Maintenance");
            testUtils.deleteApi();
        }

        try {
            String lifecycleChecklist = null;
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            String apiId = testUtils.createApi("API-120", "1.0.0", "API-120");

            // Retired --> Deprecated
            api.apisChangeLifecyclePost("Published", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
            api.apisChangeLifecyclePost("Deprecated", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
            api.apisChangeLifecyclePost("Retired", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
            api.apisChangeLifecyclePost("Deprecated", apiId, lifecycleChecklist, ifMatch, ifUnmodifiedSince);
        } catch (ApiException ae) {
            Assert.assertEquals(ae.getCode(), 400, "Response code mismatch. Retired --> Deprecated");
            testUtils.deleteApi();
        }
        testUtils.deleteApi();
    }

    @Test(description = "This testcase verifies if an API can be deleted through a REST call.")
    public void apisApiIdDeleteTest() throws ApiException {
        String apiId = testUtils.createApi("API-118", "1.0.0", "API-118");
        String ifMatch = null;
        String ifUnmodifiedSince = null;
        api.apisApiIdDelete(apiId, ifMatch, ifUnmodifiedSince);

        APIList response = apiSetup.apisGet(10, 0, null, null);
        for (int i = 0; i < response.getCount(); i++) {
            if (response.getList().get(i).getName().equals("API-118")) {
                assert (false);
            }
        }
        assert (true);
    }

    @Test(enabled = false)
    public void apisLifecyclePendingTask() throws ApiException {
        String apiId = testUtils.createApi("API-200", "1.0.0", "API-200");
        API body = api.apisApiIdGet(apiId, null, null);
        body.setWorkflowStatus("PENDING");
        String ifMatch = null;
        String ifUnmodifiedSince = null;
        API response = api.apisApiIdPut(apiId, body, ifMatch, ifUnmodifiedSince);
        System.out.println(response);
        api.apisApiIdLifecycleLifecyclePendingTaskDelete(APIID);
    }

    @Test
    public void apisLifecyclePendingTask_failureTest() throws ApiException {
        String apiId = "invalidApiID";
        try {
            api.apisApiIdLifecycleLifecyclePendingTaskDelete(apiId);
        } catch (ApiException apiException) {
            int statusCode = apiException.getCode();
            Assert.assertEquals(statusCode, 404, "status code mismatch");
        }

    }

    @Test(description = "This testcase verifies if an API can be deleted when an invalid Api Id is provided.")
    public void apisApiIdDeleteTest_failureTest() throws ApiException {
        try {
            String apiId = "invaliApi";
            String ifMatch = null;
            String ifUnmodifiedSince = null;
            api.apisApiIdDelete(apiId, ifMatch, ifUnmodifiedSince);
        } catch (ApiException ae) {
            JsonParser parser = new JsonParser();
            JsonObject responseBody = (JsonObject) parser.parse(ae.getResponseBody());
            String errorMsg = responseBody.get("message").getAsString();
            Assert.assertEquals(ae.getCode(), 404, "Response code mismatch.");
            Assert.assertEquals(errorMsg, "API not found", "Response message mismatch");
        }

    }

    @AfterClass
    public void afterClass() throws ApiException {
        APIList response = apiSetup.apisGet(10, 0, null, null);
        for (int i = 0; i < response.getCount(); i++) {
            api.apisApiIdDelete(response.getList().get(i).getId(), null, null);
        }

    }
}
